/*
 * Copyright (c) 1995-1999 Joey Hess (joey@kitenet.net)
 * All rights reserved. See COPYING for full copyright information (GPL).
 */

int Pdmenu_Action(Menu_Type *);
void Handle_Ctrl_C(void);
int NullAction(void);
