/*
 * Colors
 */

/* 
 * Copyright (c) 1995-1999 Joey Hess (joey@kitenet.net)
 * All rights reserved. See COPYING for full copyright information (GPL).
 */

/* The default colors of the parts of the screen (can be overridden in rc file) */
#define DESKTOP_FG_DEFAULT "white"
#define DESKTOP_BG_DEFAULT "blue"

#define TITLE_FG_DEFAULT "black"
#define TITLE_BG_DEFAULT "cyan"

#define BASE_FG_DEFAULT "black"
#define BASE_BG_DEFAULT "cyan"

#define MENU_FG_DEFAULT "black"
#define MENU_BG_DEFAULT "cyan"

#define SELBAR_FG_DEFAULT MENU_BG_DEFAULT
#define SELBAR_BG_DEFAULT MENU_FG_DEFAULT

#define SHADOW_FG_DEFAULT "gray"
#define SHADOW_BG_DEFAULT "black"

#define MENU_HI_FG_DEFAULT "white"
#define MENU_HI_BG_DEFAULT "cyan"

#define SELBAR_HI_FG_DEFAULT "white"
#define SELBAR_HI_BG_DEFAULT "black"

#define UNSEL_MENU_FG_DEFAULT "black"
#define UNSEL_MENU_BG_DEFAULT "lightgray"
