/*
 * General config 
 */

/* 
 * Copyright (c) 1995-1999 Joey Hess (joey@kitenet.net)
 * All rights reserved. See COPYING for full copyright information (GPL).
 */

/*
 * Comment out the following line if you don't want pdmenu to support the 
 * setenv flag ('s') to exec. This will make pdmenu a bit smaller. 
 */
#define SETENV_FLAG_OK

/*
 * Here are some more files you can edit to further configure pdmenu.
 * You can safely leave these files alone.
 */ 

/* Edit this file to change keywords used in the rc file. */
#include "rc_config.h"

/* Edit this file to change the default colors pdmenu will use. */
#include "colors.h"
